@@ -4,9 +4,7 @@ use JSON::Tiny 'decode_json';
 
 my @json = split /-{5,}/, read_file('sample.json');
 
-sub json_tiny {
-  return [ map { decode_json($_) } @json ];
-}
+sub json_tiny { return [ map { decode_json($_) } @json ]; }
 
 my $decoded = json_tiny();
 print Dumper $decoded;
